@@ -1,11 +1,11 @@
 /************* TabJDBC C++ Program Source Code File (.CPP) *************/
 /* PROGRAM NAME: TABJDBC                                               */
 /* -------------                                                       */
-/*  Version 1.2                                                        */
+/*  Version 1.3                                                        */
 /*                                                                     */
 /* COPYRIGHT:                                                          */
 /* ----------                                                          */
-/*  (C) Copyright to the author Olivier BERTRAND          2016-2017    */
+/*  (C) Copyright to the author Olivier BERTRAND          2016-2019    */
 /*                                                                     */
 /* WHAT THIS PROGRAM DOES:                                             */
 /* -----------------------                                             */
@@ -153,7 +153,7 @@ int JDBCDEF::ParseURL(PGLOBAL g, char *url, bool b)
 //	  Tabname = GetStringCatInfo(g, "Tabname", Tabname);
 		} // endif
 
-		if (trace)
+		if (trace(1))
 			htrc("server: %s Tabname: %s", url, Tabname);
 
 		// Now make the required URL
@@ -185,10 +185,10 @@ int JDBCDEF::ParseURL(PGLOBAL g, char *url, bool b)
 		} else		 // host is a URL
 			Url = PlugDup(g, server->host);
 
-		if (server->username)
+		if (!Username && server->username)
 			Username = PlugDup(g, server->username);
 
-		if (server->password)
+		if (!Password && server->password)
 			Password = PlugDup(g, server->password);
 
 		return RC_NF;
@@ -223,10 +223,10 @@ bool JDBCDEF::DefineAM(PGLOBAL g, LPCSTR am, int poff)
 	} // endif Connect
 
 	if (Url)
-		rc = ParseURL(g, Url);
-
-	if (rc == RC_FX)						// Error
-		return true;
+		if ((rc = ParseURL(g, Url)) == RC_FX) {
+			sprintf(g->Message, "Wrong JDBC URL %s", Url);
+			return true;
+		} // endif rc
 
 	Wrapname = GetStringCatInfo(g, "Wrapper", NULL);
 	return false;
@@ -305,12 +305,12 @@ TDBJDBC::TDBJDBC(PJDBCDEF tdp) : TDBEXT(tdp)
 	if (tdp) {
 		Ops.Driver = tdp->Driver;
 		Ops.Url = tdp->Url;
-		WrapName = tdp->Wrapname;
+		Wrapname = tdp->Wrapname;
 		Ops.User = tdp->Username;
 		Ops.Pwd = tdp->Password;
 		Ops.Scrollable = tdp->Scrollable;
 	} else {
-		WrapName = NULL;
+		Wrapname = NULL;
 		Ops.Driver = NULL;
 		Ops.Url = NULL;
 		Ops.User = NULL;
@@ -328,7 +328,7 @@ TDBJDBC::TDBJDBC(PTDBJDBC tdbp) : TDBEXT(tdbp)
 {
 	Jcp = tdbp->Jcp;            // is that right ?
 	Cnp = tdbp->Cnp;
-	WrapName = tdbp->WrapName;
+	Wrapname = tdbp->Wrapname;
 	Ops = tdbp->Ops;
 	Prepared = tdbp->Prepared;
 	Werr = tdbp->Werr;
@@ -470,7 +470,7 @@ bool TDBJDBC::MakeInsert(PGLOBAL g)
 	else
 		Prepared = true;
 
-	if (trace)
+	if (trace(33))
 		htrc("Insert=%s\n", Query->GetStr());
 
 	return false;
@@ -553,38 +553,47 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 {
 	bool rc = true;
 
-	if (trace)
+	if (trace(1))
 		htrc("JDBC OpenDB: tdbp=%p tdb=R%d use=%d mode=%d\n",
 		     this, Tdb_No, Use, Mode);
 
 	if (Use == USE_OPEN) {
-		/*******************************************************************/
-		/*  Table already open, just replace it at its beginning.          */
-		/*******************************************************************/
-		if (Memory == 1) {
-			if ((Qrp = Jcp->AllocateResult(g)))
-				Memory = 2;            // Must be filled
-			else
-				Memory = 0;            // Allocation failed, don't use it
+		if (Mode == MODE_READ || Mode == MODE_READX) {
+			/*****************************************************************/
+			/*  Table already open, just replace it at its beginning.        */
+			/*****************************************************************/
+			if (Memory == 1) {
+				if ((Qrp = Jcp->AllocateResult(g, this)))
+					Memory = 2;            // Must be filled
+				else
+					Memory = 0;            // Allocation failed, don't use it
 
-		} else if (Memory == 2)
-			Memory = 3;              // Ok to use memory result
+			} else if (Memory == 2)
+				Memory = 3;              // Ok to use memory result
 
-		if (Memory < 3) {
-			// Method will depend on cursor type
-			if ((Rbuf = Jcp->Rewind(Query->GetStr())) < 0)
-				if (Mode != MODE_READX) {
-					Jcp->Close();
-				  return true;
-			  } else
-				  Rbuf = 0;
+			if (Memory < 3) {
+				// Method will depend on cursor type
+				if ((Rbuf = Query ? Jcp->Rewind(Query->GetStr()) : 0) < 0)
+					if (Mode != MODE_READX) {
+						Jcp->Close();
+						return true;
+					} else
+						Rbuf = 0;
 
-		} else
-			Rbuf = Qrp->Nblin;
+			} else
+				Rbuf = Qrp->Nblin;
+
+			CurNum = 0;
+			Fpos = 0;
+			Curpos = 1;
+		} else if (Mode == MODE_UPDATE || Mode == MODE_DELETE) {
+			// new update coming from a trigger or procedure
+			Query = NULL;
+			SetCondFil(NULL);
+			Qrystr = To_Def->GetStringCatInfo(g, "Query_String", "?");
+		} else {  //if (Mode == MODE_INSERT)
+		} // endif Mode
 
-		CurNum = 0;
-		Fpos = 0;
-		Curpos = 1;
 		return false;
 	} // endif use
 
@@ -596,19 +605,23 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 	/*  drivers allowing concurency in getting results ???               */
 	/*********************************************************************/
 	if (!Jcp)
-		Jcp = new(g)JDBConn(g, this);
+		Jcp = new(g)JDBConn(g, Wrapname);
 	else if (Jcp->IsOpen())
 		Jcp->Close();
 
-	if (Jcp->Open(&Ops) == RC_FX)
+	if (Jcp->Connect(&Ops))
 		return true;
 	else if (Quoted)
 		Quote = Jcp->GetQuoteChar();
 
+	if (Mode != MODE_READ && Mode != MODE_READX)
+		if (Jcp->SetUUID(g, this))
+			PushWarning(g, this, 1);
+
 	Use = USE_OPEN;       // Do it now in case we are recursively called
 
 	/*********************************************************************/
-	/*  Make the command and allocate whatever is used for getting results.                   */
+	/* Make the command and allocate whatever is used for getting results*/
 	/*********************************************************************/
 	if (Mode == MODE_READ || Mode == MODE_READX) {
 		if (Memory > 1 && !Srcdef) {
@@ -625,7 +638,7 @@ bool TDBJDBC::OpenDB(PGLOBAL g)
 				} else if (n) {
 					Jcp->m_Rows = n;
 
-					if ((Qrp = Jcp->AllocateResult(g)))
+					if ((Qrp = Jcp->AllocateResult(g, this)))
 						Memory = 2;            // Must be filled
 					else {
 						strcpy(g->Message, "Result set memory allocation failed");
@@ -767,7 +780,7 @@ bool TDBJDBC::ReadKey(PGLOBAL g, OPVAL op, const key_range *kr)
 		Mode = MODE_READ;
 	} // endif's op
 
-	if (trace)
+	if (trace(33))
 		htrc("JDBC ReadKey: Query=%s\n", Query->GetStr());
 
 	rc = Jcp->ExecuteQuery((char*)Query->GetStr());
@@ -783,7 +796,7 @@ int TDBJDBC::ReadDB(PGLOBAL g)
 {
 	int  rc;
 
-	if (trace > 1)
+	if (trace(2))
 		htrc("JDBC ReadDB: R%d Mode=%d\n", GetTdb_No(), Mode);
 
 	if (Mode == MODE_UPDATE || Mode == MODE_DELETE) {
@@ -836,7 +849,7 @@ int TDBJDBC::ReadDB(PGLOBAL g)
 
 	} // endif placed
 
-	if (trace > 1)
+	if (trace(2))
 		htrc(" Read: Rbuf=%d rc=%d\n", Rbuf, rc);
 
 	return rc;
@@ -897,7 +910,7 @@ int TDBJDBC::WriteDB(PGLOBAL g)
 
 	Query->RepLast(')');
 
-	if (trace > 1)
+	if (trace(2))
 		htrc("Inserting: %s\n", Query->GetStr());
 
 	rc = Jcp->ExecuteUpdate(Query->GetStr());
@@ -925,7 +938,7 @@ int TDBJDBC::DeleteDB(PGLOBAL g, int irc)
 			AftRows = Jcp->m_Aff;
 			sprintf(g->Message, "%s: %d affected rows", TableName, AftRows);
 
-			if (trace)
+			if (trace(1))
 				htrc("%s\n", g->Message);
 
 			PushWarning(g, this, 0);    // 0 means a Note
@@ -946,14 +959,14 @@ void TDBJDBC::CloseDB(PGLOBAL g)
 	if (Jcp)
 		Jcp->Close();
 
-	if (trace)
+	if (trace(1))
 		htrc("JDBC CloseDB: closing %s\n", Name);
 
 	if (!Werr && 
 		(Mode == MODE_INSERT || Mode == MODE_UPDATE || Mode == MODE_DELETE)) {
 		sprintf(g->Message, "%s: %d affected rows", TableName, AftRows);
 
-		if (trace)
+		if (trace(1))
 			htrc("%s\n", g->Message);
 
 		PushWarning(g, this, 0);    // 0 means a Note
@@ -970,6 +983,7 @@ void TDBJDBC::CloseDB(PGLOBAL g)
 JDBCCOL::JDBCCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i, PCSZ am)
 	     : EXTCOL(cdp, tdbp, cprec, i, am)
 {
+	uuid = false;
 } // end of JDBCCOL constructor
 
 /***********************************************************************/
@@ -977,6 +991,7 @@ JDBCCOL::JDBCCOL(PCOLDEF cdp, PTDB tdbp, PCOL cprec, int i, PCSZ am)
 /***********************************************************************/
 JDBCCOL::JDBCCOL(void) : EXTCOL()
 {
+	uuid = false;
 } // end of JDBCCOL constructor
 
 /***********************************************************************/
@@ -985,12 +1000,11 @@ JDBCCOL::JDBCCOL(void) : EXTCOL()
 /***********************************************************************/
 JDBCCOL::JDBCCOL(JDBCCOL *col1, PTDB tdbp) : EXTCOL(col1, tdbp)
 {
+	uuid = col1->uuid;
 } // end of JDBCCOL copy constructor
 
 /***********************************************************************/
-/*  ReadColumn: when SQLFetch is used there is nothing to do as the    */
-/*  column buffer was bind to the record set. This is also the case    */
-/*  when calculating MaxSize (Bufp is NULL even when Rows is not).     */
+/*  ReadColumn: retrieve the column value via the JDBC driver.         */
 /***********************************************************************/
 void JDBCCOL::ReadColumn(PGLOBAL g)
 {
@@ -1117,7 +1131,7 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 {
 	bool rc = false;
 
-	if (trace)
+	if (trace(1))
 		htrc("JDBC OpenDB: tdbp=%p tdb=R%d use=%d mode=%d\n",
 		this, Tdb_No, Use, Mode);
 
@@ -1134,11 +1148,11 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 	/*  drivers allowing concurency in getting results ???               */
 	/*********************************************************************/
 	if (!Jcp) {
-		Jcp = new(g) JDBConn(g, this);
+		Jcp = new(g) JDBConn(g, Wrapname);
 	} else if (Jcp->IsOpen())
 		Jcp->Close();
 
-	if (Jcp->Open(&Ops) == RC_FX)
+	if (Jcp->Connect(&Ops))
 		return true;
 
 	Use = USE_OPEN;       // Do it now in case we are recursively called
@@ -1152,8 +1166,9 @@ bool TDBXJDC::OpenDB(PGLOBAL g)
 	/*  Get the command to execute.                                      */
 	/*********************************************************************/
 	if (!(Cmdlist = MakeCMD(g))) {
-		Jcp->Close();
-		return true;
+		// Next lines commented out because of CHECK TABLE
+		//Jcp->Close();
+		//return true;
 	} // endif Query
 
 	Rows = 1;
@@ -1173,7 +1188,7 @@ int TDBXJDC::ReadDB(PGLOBAL g)
 		else
 			Query->Set(Cmdlist->Cmd);
 
-		if ((rc = Jcp->ExecSQLcommand(Query->GetStr())) == RC_FX)
+		if ((rc = Jcp->ExecuteCommand(Query->GetStr())) == RC_FX)
 			Nerr++;
 
 		if (rc == RC_NF)
@@ -1184,8 +1199,10 @@ int TDBXJDC::ReadDB(PGLOBAL g)
 		Fpos++;                // Used for progress info
 		Cmdlist = (Nerr > Mxr) ? NULL : Cmdlist->Next;
 		return RC_OK;
-	} else
+	} else {
+		PushWarning(g, this, 1);
 		return RC_EF;
+	}	// endif Cmdlist
 
 } // end of ReadDB
 
