@@ -13,7 +13,7 @@ FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
-51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+51 Franklin Street, Suite 500, Boston, MA 02110-1335 USA
 
 *****************************************************************************/
 
@@ -56,15 +56,16 @@ ib_wqueue_free(
 /*===========*/
 	ib_wqueue_t*	wq);	/*!< in: work queue */
 
-/** Add a work item to the queue.
-@param[in,out]	wq		work queue
-@param[in]	item		work item
-@param[in,out]	heap		memory heap to use for allocating list node
-@param[in]	wq_locked	work queue mutex locked */
+/****************************************************************//**
+Add a work item to the queue. */
 UNIV_INTERN
 void
-ib_wqueue_add(ib_wqueue_t* wq, void* item, mem_heap_t* heap,
-	      bool wq_locked = false);
+ib_wqueue_add(
+/*==========*/
+	ib_wqueue_t*	wq,	/*!< in: work queue */
+	void*		item,	/*!< in: work item */
+	mem_heap_t*	heap);	/*!< in: memory heap to use for allocating the
+				list node */
 
 /** Check if queue is empty.
 @param wq wait queue
@@ -88,7 +89,7 @@ ib_wqueue_timedwait(
 /*================*/
 					/* out: work item or NULL on timeout*/
 	ib_wqueue_t*	wq,		/* in: work queue */
-	ulint		wait_in_usecs); /* in: wait time in micro seconds */
+	ib_time_t	wait_in_usecs); /* in: wait time in micro seconds */
 
 /********************************************************************
 Return first item on work queue or NULL if queue is empty
@@ -106,16 +107,14 @@ ib_wqueue_len(
 /*==========*/
 	ib_wqueue_t*	wq);		/*<! in: work queue */
 
-/** Work queue */
-struct ib_wqueue_t
-{
-	/** Mutex protecting everything */
-	ib_mutex_t	mutex;
-	/** Work item list */
-	ib_list_t*	items;
-	/** event we use to signal additions to list;
-	os_event_set() and os_event_reset() are protected by the mutex */
-	os_event_t	event;
+
+/* Work queue. */
+struct ib_wqueue_t {
+	ib_mutex_t		mutex;	/*!< mutex protecting everything */
+	ib_list_t*	items;	/*!< work item list */
+	os_event_t	event;	/*!< event we use to signal additions to list;
+				os_event_set() and os_event_reset() are
+				protected by ib_wqueue_t::mutex */
 };
 
 #endif
