@@ -256,7 +256,7 @@ int main(int argc __attribute__((unused)), char *argv[])
 #endif
 
   long_buffer= malloc(LONG_BUFFER_SIZE + LSN_STORE_SIZE * 2 + 2);
-  load_defaults("my", load_default_groups, &argc, &argv);
+  load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   default_argv= argv;
   get_options(&argc, &argv);
 
